/** Datafield Class.
 * This Class is for storing Strings, which represents the different Unit-components
 * There are different options for the Datafields. See below
 */

import java.lang.*;
import java.awt.*;
import java.util.*;
import java.lang.Math.*;

public class DataField extends Object{

   private String data;                         // String containing the Data to be displayed
   private Rectangle rect;                      // rectangle-Area for the Paintings
                                                // x and y are the translation-vector for painting
   private int x,y;
   private boolean activated = false;           // true if the field is activated by the User 

// Variables containing drawing information:
   private int visibility = 0;			// 0 = never
						// 1 = if activated
						// 2 = always
   private Color activeColor;
   private Color inactiveColor;
   private int colorMode = 0;			// 0 = use the colors given by active/inactiveColor

   private int shape = 0;			// 0 = no outline. use Color for Text
   						// 1 = box shape
						// 2 = rounded box shape
						// 3 = circle shape
   private int line = 0;			// 0 = no line from Datafield to center of Unit
						// 1 = solid Line from Datafield to center of Unit
   private boolean sticky = false;		// false: Datafield is moved if Unit is moved
						// true: Datafield is sticked to its position


   public DataField( String data) {
       this.data = new String(data);
       x = 0;
       y = 0;
       rect = new Rectangle(0,0,10,10);

   }

   public Properties getProperties(Properties props, int i) {
       // i is used as an index to allow mulptiple datafields to write properties in one
       // Property props.
 
       props.setProperty ("Visibility"+i, Integer.toString(visibility));
       props.setProperty ("activeColor"+i, Integer.toString(activeColor.getRGB()));
       props.setProperty ("inactiveColor"+i, Integer.toString(inactiveColor.getRGB()));
       props.setProperty ("coloMode"+i, Integer.toString(colorMode));
       props.setProperty ("shape"+i, Integer.toString(shape));
       props.setProperty ("line"+i, Integer.toString(line));
       props.setProperty ("sticky"+i, String.valueOf(sticky));

       return props;
   }

   public void setProperties(Properties props, int i) {
       // i is used as an index to allow mulptiple datafields to read properties from one   
       // Property props.

	//System.out.println("Visibility"+i+"  ="+props.getProperty("Visibility"+i));

       visibility = Integer.parseInt(props.getProperty("Visibility"+i));
       activeColor = new Color(Integer.parseInt(props.getProperty("activeColor"+i)));
       inactiveColor = new Color(Integer.parseInt(props.getProperty("inactiveColor"+i)));
       colorMode = Integer.parseInt(props.getProperty("coloMode"+i));
       shape = Integer.parseInt(props.getProperty("shape"+i));
       line = Integer.parseInt(props.getProperty("line"+i));
       sticky = props.getProperty("sticky"+i).equals("true"); 
   }


   public void translate(int x, int y) {
       //rect.translate(x, y);
       this.x = this.x +x;
       this.y = this.y +y; 
      
   }
   public void customTranslate(int x, int y) {
       if (!sticky) {
           this.translate(x, y);
       }
   }
   public void setX(int x) {
	this.x = x;
   }
   public void setY(int y) {
	this.y = y;
   }
   public int getX() { 
	return x;
   }
   public int getY() {
       return y;
   }

   public void paintPS(PSGr g, Properties p) {
       if (this.isVisible()) {
	   int d = Math.max(rect.width, rect.height);
           int r = Math.min(rect.width, rect.height)/2;	

           if (activated) {
               g.setColor(activeColor);
           } else {
               g.setColor(inactiveColor);
           }
           if (shape == 1) {
               g.fillRect(rect.x, rect.y, rect.width, rect.height);
           } else if (shape == 2) {
               g.fillRoundRect(rect.x, rect.y, rect.width, rect.height, r, r);
           } else if (shape == 3) {
               g.fillOval(x - d/2, y - d/2, d, d);
           }
           g.setColor(Color.darkGray);
           if (shape == 1) {
               g.drawRect(rect.x, rect.y, rect.width, rect.height);
           } else if (shape == 2) {
               g.drawRoundRect(rect.x, rect.y, rect.width, rect.height, r, r);
           } else if (shape == 3) {
               g.drawOval(x - d/2, y - d/2, d, d);
           }
	   g.setColor(Color.black);
	   g.drawString(data, rect.x+3, rect.y+rect.height-3); 

       }
   }

   public void paintComponent(Graphics g, Properties p, boolean hasFocus) {
       if (this.isVisible()) {
	   if (activated) {
  	       g.setColor(activeColor);
	   } else {
	       g.setColor(inactiveColor);
	   }
	   FontMetrics fm = g.getFontMetrics();

	   //Rectangle fontRect = (Rectangle) fm.getStringBounds(data,g);
	   java.awt.geom.Rectangle2D fontRect = fm.getStringBounds(data,g);
	   int w = (int) fontRect.getWidth()+6;
	   int h = (int) fontRect.getHeight()+6;
	   if (w<10) w = 10;
	   if (h<10) h = 10;
	   int d = w;
	   if (h>w) d = h;

	   // resize rect.

	   rect.setLocation(this.x - w/2, this.y - h/2);
	   rect.setSize(w,h);

	   //System.out.println(data+"   "+fontRect.toString());

	   if (shape == 1) {
               g.fillRect(rect.x, rect.y, rect.width, rect.height);
           } else if (shape == 2) {
	       g.fillRoundRect(rect.x, rect.y, rect.width, rect.height, h/2, h/2);
           } else if (shape == 3) {
               g.fillOval(x - d/2, y - d/2, d, d);
	   }
	   if (hasFocus) {
	       g.setColor(Color.darkGray);
               if (shape == 1) {
                   g.drawRect(rect.x, rect.y, rect.width, rect.height);
		   g.drawRect(rect.x-1, rect.y-1, rect.width+2, rect.height+2);
               } else if (shape == 2) {
                   g.drawRoundRect(rect.x, rect.y, rect.width, rect.height, h/2, h/2);
		   g.drawRoundRect(rect.x-1, rect.y-1, rect.width+2, rect.height+2, h/2+1, h/2+1);
               } else if (shape == 3) {
                   g.drawOval(x - d/2, y - d/2, d, d);
                   g.drawOval(x - d/2-1, y - d/2-1, d+2, d+2);
               }
	   }

	   g.setColor(Color.black);
           g.drawString(data, rect.x+3, rect.y+rect.height-3);
       }
   }

   public Rectangle getBorder() {
       if (isVisible()) {
           return rect;
       }
       return null;
   }


   public String getData() {
       return data;
   }

   public String toString() {
       return data;
   }

   public void setAlwaysVisible() {
      visibility = 2;
   }


   public boolean isVisible() {
       if (visibility == 2) return true;		// always visible
       if (visibility == 0) return false;		// never visible
       return activated;				// visible if activated
   }

   public void setActivation(boolean active) {
       activated = active;
   }

   public boolean contains(Point p) {
      return rect.contains(p);
   }

/*
   public boolean MouseClicked(java.awt.event.MouseEvent evt) {
      boolean handled = false;

      // the Datafield must be visible !
      if (this.isVisible()) {
          //System.out.println("test Datafield");
          if (rect.contains(evt.getPoint())) {
             //System.out.println("HIT !!!"+evt.toString());
             handled = true;
          }
      }

      return handled;
   }
*/

}
